@@ -7,6 +7,8 @@ import os
 import pandas as pd
 from abc import ABC, abstractmethod
 import numpy as np
+import requests
+from bs4 import BeautifulSoup
 
 nifty_next_50 = ["ABBOTINDIA", "ACC", "ADANIENT", "ADANIGREEN", "ADANITRANS", "ALKEM", "AMBUJACEM", "APOLLOHOSP", "AUROPHARMA", "DMART", "BAJAJHLDNG", "BANDHANBNK", "BERGEPAINT", "BIOCON", "BOSCHLTD", "CADILAHC", "COLPAL", "DABUR", "DLF", "GAIL", "GODREJCP", "HAVELLS", "HDFCAMC", "HINDPETRO", "ICICIGI", "ICICIPRULI", "IGL", "NAUKRI", "INDIGO", "JUBLFOOD", "LTI", "LUPIN", "MARICO", "MOTHERSUMI", "MRF", "MUTHOOTFIN", "NMDC", "PETRONET", "PIDILITIND", "PEL", "PGHH", "PNB", "SBICARD", "SIEMENS", "TORNTPHARM", "UBL", "MCDOWELL-N", "VEDL", "YESBANK"]
 
@@ -231,7 +233,10 @@ class Strategy52WHigh(Strategy):
         pending_tickers = self.getTickerList(pending = True)
         processed_tickers = self.getTickerList(processed = True)
         if len(pending_tickers) != 0:
-            sample_tickers = random.sample(pending_tickers, min(50, len(pending_tickers)))
+            if self.save:
+                sample_tickers = random.sample(pending_tickers, min(50, len(pending_tickers)))
+            else:
+                sample_tickers = pending_tickers
             for ticker in sample_tickers:
                 try:
                     df = self.processStrategy(ticker)
@@ -285,4 +290,73 @@ class Strategy52WHigh(Strategy):
 
             return message
         else:
-            return result
\ No newline at end of file
+            return result
+        
+class ScreenerData:
+    
+    def __init__(self, url = None):
+        self.url = url
+        
+    @property
+    def url(self):
+        return self._url
+    
+    @url.setter
+    def url(self, value):
+        if url is not None:
+            self._url = value
+        
+    def get(self):
+        page = requests.get(self.url)
+        soup = BeautifulSoup(page.content, "html.parser")
+        result = [x.find("tbody") for x in soup.find_all("table")]
+        
+        data = []
+        # Headers
+        headers = result[0].find("tr").find_all("th")
+        headers = [y.find('a').children for y in headers]
+        headers = [next(z).strip() for z in headers]
+
+        # Data
+        comps = result[0].find_all("tr")[1:]
+        for comp in comps:
+            details = comp.find_all("td")
+            i = 0
+            temp = {}
+            for detail in details:
+                if detail.find('a') is not None:
+                    value = next(detail.find('a').children).strip()
+                else:
+                    value = next(detail.children).strip()
+                try:
+                    value = float(value)
+                except:
+                    pass
+                temp[headers[i]] = value
+                i += 1
+            data.append(temp)
+
+        data = pd.DataFrame(data)
+        return data
+        
+class MagicFormulaStrategy:
+    
+    def screener(self, formatMessage = True):
+        url = 'https://www.screener.in/screens/450341/Magic-Formula/'
+        obj = ScreenerData(url)
+        data = obj.get()
+        data = data.sort_values(by = ['Earnings Yield', 'ROCE'], ascending = False)
+        data.reset_index( inplace = True, drop = True)
+
+        # Strategy
+        amount_per_stock = 10000
+        data['Quantity'] = round(amount_per_stock/data['CMP'],0)
+        
+        if formatMessage:
+            message = '<b>Magic Formula Portfolio</b>\nInvest: 10K per ticker\n\n'
+            for stock in data.to_dict(orient = 'record'):
+                message += '<b>' + stock['Name'] + '</b>\n'
+                message += 'LTP ' + str(round(stock['CMP'], 2)) + '\n'
+                message += 'Quantity: ' + str(round(stock['Quantity'], 2)) + '\n\n'
+            return message
+        return data
\ No newline at end of file
